''***************************************
''*  z8Emulator v1.0  for TINY2012      *
''*  Author: Uwe Nickel                 *
''*  Copyright (c) 2012                 *
''*  See end of file for terms of use.  *
''***************************************
'Stand: Alpha 1.0:
' Tatstaurmatrix rudimentr
' IRQ ausgeschaltet

'Tastaturabfrage: Treiber setzt Zellen im RAM entspr. Tastaturmatrix 
'===============================================================================================================================
' Register Z8 im HUB-RAM
' Speicher Z8 im HUB-RAM
' Flagtabelle im HUB-RAM
' div. Kommandos im HUB-RAM
'================================================================================================================================
'
'

CON

_clkmode        = xtal1 + pll16x
_xinfreq        = 5_000_000
'--------------------------------------------------------------------------------
' Festlegungen 
'--------------------------------------------------------------------------------

' TINY Memory-Map ...............................................................
 romstart       = $0000                        '1. ROM-Adresse ia. gleich 1. Adresse Programmemory
 romend         = $1000                        'Wert entsprechend ROM-Filelnge setzen!
                                               '2k-System: 1000,  4k-System: 2FFF
 ramstart       = $E000
 ramend         = $FFFF                        'Letzte RAM-Adresee entspricht Ende Programmemory
 bwsstart       = $FE00
 bwsend         = $FFFF
 keystart       = $7F00
 keyend         = $7F0F
 dmemstart      = $E000                        '1.Adresse Datamemory, ggf. gleich PM-Adressen
 dmemend        = $FFFF
 calccmdreg     = 0                            'Registernummer zur bergabe an Berechnungs-Cog
 
 ram_len        = ramend - ramstart +1
 rom_len        = romend - romstart
 key_len        = keyend - keystart  
 lenpmem        = ram_len + rom_len 
 
' Zuweisung externe Gerte (IO-Port's) zu Proprller-Pins ............................................. 
  P2startpin    = 8            'niedrigster Portpin fr P2-Hardware-Emulation
  P30           = 0
  P31           = 1
  P32           = 2
  P33           = 3
  P34           = 4
  P35           = 5
  P36           = 6
  P37           = 7          
  vgapin        = 16           'VGA - ab Pin16 acht Pins                            
  kbdata        = 26           'Tastatur-Daten                                              
  kbclock       = 27           'Tastatur-Clock                                              
 ' TxPin          = 30           'Portpin serielle Ausgabe                                   
 ' DtrPin         = 25           'Portpin DTR                                                
 ' RxPin          = 31           'Portpin serielle Eingabe                                   
 ' CtsPin         = 24           'Portpin CTS                                                
 ' sound_l        = 13           'Soundausgabe links                                         
 ' sound_r        = 12           'Soundausgabe rechts                                        
                                                                                      
' Tastatur ......................................................................
lockxx          = %0_000_000   'Lock- Setup
autoxx          = %00_00000    'Autorepeat- Setup
'................................................................................
''      lockxx  - bit 6 disallows shift-alphas (case set soley by CapsLock)        
''                bits 5..3 disallow toggle of NumLock/CapsLock/ScrollLock state   
''                bits 2..0 specify initial state of NumLock/CapsLock/ScrollLock   
''               (eg. %0_001_100 = disallow ScrollLock, NumLock initially 'on')    
''      autoxx  - auto-repeat setup                                                
''                bits 6..5 specify delay (0=.25s, 1=.5s, 2=.75s, 3=1s)            
''                bits 4..0 specify repeat rate (0=30cps..31=2cps)                 
''               (eg %01_00000 = .5s delay, 30cps repeat)                          

' Bildschirm (Memory etc.).......................................................
 lencol         = $12C/2               'Gre Farbspeicher in Word                            
 lenbmp         = 300'1200'2400        'Gre Bitmap-Speicher in Longs (original 4800)        
 lenx           = bwsend - bwsstart +1 'Gre Bildspeicher TINY in Byte                                
 stdcol         = $FF_09               'Vordergrundfarbe(wei)_ Hintergrundfarbe(blau)                 
 stdmodus       = $01_00_00_00         'Ausgabemodus: xp_yp_zxzy_x/x/x/x/cls/im/tm/zr                             
 '..................................................................................................
''           xp - X-Position auf Schirm in Spalten (1Spalte = 1long)
''           yp - Y-Position auf Schirm in Zeilen  (1Zeile  = 20long's)
''           zx - X-Zoomfactor ("1" or "2")
''           zy - Y-Zoomfactor ()
''          cls - Clear Screen vor Ausgabe neuer Werte ("0"- no CLS, "1"- CLS) 
''           im - Invers-Mode fr Bildschirmspeicher ("0"  or "1")
''           tm - Tile-Mode fr VGA-Driver, y-Direction: lines per Tile, Value of equal lines
''                                          Mode "0" :   8               4
''                                          Mode "1" :   16              2
''           zr - unbenutzte 1.Zeile vom TINY-Screen: show "1" or hide "0"
 
VAR   
    long cog, z8cog

' Emulator..............................................................................
    long z8bus
         
' VGA etc...............................................................................
    long  sync
    long  vgamodi
    long  bwsstart_adr
    long  mode_base  
    long  shift_adr     

    long  xyzdata
    long  calccmdadr
    byte  calccmd
    
OBJ
    intio         : "intio-z8emu"
    moni          : "vga-z8emu_uwevga"
    calcvga       : "calc-z8emu_uwevga"   
    kb            : "keybdrv-z8Cog_xxx"
 
PUB Main      

'HUB-Ram Speicheradressen     
 z8reg_adr   := @REGx    
 rombaseadr  := @ROMx
 rambaseadr  := @RAMx
 keybaseadr  := @KEYx
 shift_adr   := @RAMx + ram_len-1
 bwsstart_adr:= @RAMx + (bwsstart -ramstart)
 flag8_calc  := @flag8
 flag16_calc := @flag16
 ldxx_adr    := @ldxx
 ldcx_adr    := @ldcx
 ldIRR5_adr  := @ldIRR5
 do_res_adr  := @do_res
 do_int_adr  := @do_int
 daadd_adr   := @daadd
 dasub_adr   := @dasub
 tcmx_adr    := @tcmx
 grp_Fx_adr  := @grp_Fx
 cond_adr    := @cond
 flag_adr    := @flagtable
 grp_jmp_adr := @grp_jmp
 x0_adr      := @x0_jmp

 calccmdadr  := @REGx + calccmdreg
 
 
'Initialisierungs- und Standardwerte     
 vgamodi       := stdmodus                      'Init-Wert Darstellungsmodus
 mode_base     := (stdmodus >> 1) & 1           'Tile-Mode fr VGA-Treiber       
 xyzdata       := stdcol                        'Init-Wert Farben fr Vorder-und Hintergrund

'VGA-Treiber starten       
     moni.start (vgapin,@col_base,@bitmap_base,@mode_base,@sync)
    
'Berechnungs_Cog starten         
     calcvga.start(bwsstart_adr,lenx,@bitmap_base,lenbmp,@col_base,lencol,@vgamodi,calccmdadr,@xyzdata)
         
'Keyboard starten       
     kb.startx (kbdata, kbclock,lockxx,autoxx,@KEYx,shift_adr)                              
     

'Interrupt und I/O starten
     intio.start(@REGx,P2startpin,P30,P31,P32,P33,P34,P35,P36,P37)
       
'TINY-Cog starten
     z8cog := cognew(@START,@z8bus) + 1                                      


DAT
'....LMM- Programmteile.................................................................................                                 
flag8             mov temp, #0
                  test resultx, #$80       wz     'S-Flag   ok
                  muxnz temp, #%0010_0000
                  test resultx, #$100      wz     'C-Flag   ok
                  muxnz temp, #%1000_0000
                  and resultx, #$FF
                  cmp resultx, #$00        wz     'Z-Flag   ok
                  muxz temp, #%0100_0000
          '.......................................................
                  and val1, #$7F                  'V-Flag 
                  and val2, #$7F
                  call #calcx
                  test resultx, #$80       wz     'Test ob Carry-In nach bit7
         if_z     mov temp2,#%1000_0000 
         if_nz    mov temp2, #%0000_0000
                  xor temp2, temp                 'Carry-In xor Carry-Out -> V
                  test temp2, #$80         wz
                  muxz temp, #%0001_0000
         '........................................................
                  and val1, #$0F                  'H-Flag 
                  and val2, #$0F
                  call #calcx
                  test resultx, #$10       wz
                  muxnz temp, #%0000_0100
          '.......................................................        
                  and temp, flag_mask         
                  andn FLAG, flag_mask  
                  or FLAG, temp
                  mov z8reg_ptr, z8reg_adr                          
                  add z8reg_ptr, #$FC
                  wrbyte FLAG, z8reg_ptr 
                  jmp #calc_end '                 'zurck
                  
'.........................................................................................................                
flag16            mov temp, #0 
                  test resultx, v_16       wz     'S-Flag 
                  muxnz temp, #%0010_0000     
                  test resultx, c_16       wz     'C-Flag
                  muxnz temp, #%1000_0000
                  and resultx, adr_mask
                  cmp resultx, #$00        wz     'Z-Flag
                  muxz temp, #%0100_0000
          '......................................................
                  xor resultx, val1               'H-Flag   
                  test resultx, #$100      wz
                  muxnz temp, #%0000_0100
          '......................................................   
'                  shl val1, #16                   'V-Flag 
 '                shl val2, #16                    'wird nicht bentigt bei Z8
 '                 call #calcx
 '                 muxc temp,#%0001_0000 
              
                  and temp, flag_mask
                  andn FLAG, flag_mask
                  or FLAG, temp
                  mov z8reg_ptr, z8reg_adr                          
                  add z8reg_ptr, #$FC
                  wrbyte FLAG, z8reg_ptr
                  
                  mov regnr, dst_regnr
                  mov regval, tempval
                  call #set_IRR
                  jmp #PCinc                        'zurck
'................................................................                  
ldxx              mov temp3, nval
                  shr nval, #8
                  mov sd_typ, #1
                  mov mode, #1
                  call #r_left_x
                  mov sd_typ, #2
                  mov mode, #1
                  call #r_right_y
                  and temp3, #$FF       
                  add val2, temp3
                  mov regnr, val2
                  mov sd_typ, #2
                  call #get_REGval
                  cmp cmd_nr, #$0C
         if_z     jmp #dst_end
                  mov regnr, src_regnr
                  mov regval, val1
                  jmp #set_end
'...................................................................
ldcx              ror ival, #8                        'absolute Registernummer                                                                              
                  mov regnr, ival                     'nach <regnr>                                                                                         
                  call #get_Irr                       'Adresse aus Doppelregister holen   (Irr)                                                             
                  mov tempval, PC                     'PC retten                                                                                            
                  mov PC, regval                      'Adresse nach PC                                                                                      
                  cmp cmd_grp, #2               wz                                                                                                          
          if_z    jmp #ldcci                                                                                                                                
                  sub regnr, #1                       'Doppelregisternummer auf 1.Register zurcksetzen                                                     
                  call #inc_Irr                       'incrementiere Adresse und nach Doppelregister zurckschreiben                                        
                  mov regnr, dst_regnr                '                                                                                                     
                  mov regval, val1                    '<val1> ist Registernummer (Ir)                                                                       
                  add regval, #1                      'inkrementiere Ir                                                                                     
                  call #set_REGval                    'und nach Register zurckschreiben                                                                    
                  mov regnr, val1                     '<val1> ist Registernummer (Ir)                                                                       
                  mov dst_regnr, val1                                                                                                                       
                  call #get_REGval                    'Registerinhalt nach <regval>                                                                         
                  mov val1, regval                                                                                                                          
                  jmp #ldcci         
'...................................................................
ldIRR5    
                  mov regnr, dst_regnr
                  mov regval, src_regnr
                  jmp #set_end

'..............................................................................................
do_res            mov PC, #$0C                     'Startwerte laden
                  mov FLAG, #0                     'FLAG, 
                  mov RP, #0                       'Registerpointer
                  mov regnr, #$FC                  ' .
                  mov regval, FLAG                 ' .
                  call #set_REGval                 'setzen
                  mov regval, RP
                  call #set_incREGval
                  mov SP, #$100                     'Stackpointer
                  call #set_SP                     'setzen      
                
                  mov regval, #$FF                 'alle bits auf Eingang im
                  mov regnr, #$F6                  'P2-Mode-Register
                  call #set_REGval                 'setzen
                  mov regval, #%01001101           'Standardwerte P01M
                  mov regnr, #$F8                  'P01-Mode-Register
                  call #set_REGval                 'setzen
                  
                  mov IPR, #0                      'Interruptregister
                  mov IMR, #0                      ' .
                  mov IRQ, #0                      ' .
                  mov regnr, #$F9
                  mov regval, IPR
                  call #set_REGval
                  mov regval, IRQ
                  call #set_incREGval
                  mov regval, IMR
                  call #set_incREGval
                  mov TMR, #0                      'alle Timerregister
                  mov T1, #0                       ' .
                  mov PRE1, #0                     ' .
                  mov T2, #0                       ' .
                  mov PRE2, #0                     ' .
                  mov regval, TMR                  ' .
                  mov regnr, #$F1                  ' .
                  mov regval, TMR                  ' .
                  call #set_REGval                 ' .
                  mov regval, T1                   ' .
                  call #set_incREGval              ' . 
                  mov regval, PRE1                 ' .
                  call #set_incREGval              ' .
                  mov regval, T2                   ' .
                  call #set_incREGval              ' .
                  mov regval, PRE2                 ' .
                  call #set_incREGval              'setzen

                  mov mode, #0    
                  jmp #cmd1
'.............................................................................................
do_int            andn IMR, #%10000000             'globale Interruptsperre 
                  mov z8reg_ptr, z8reg_adr         'im Register FB                     
                  add z8reg_ptr, #$FB
                  wrbyte IMR, z8reg_ptr            'setzen
                  
                  call #PC2ST                      'PC retten
                  mov temp, FLAG                   'FLAG                                          
                  call #ST_write                   'retten
{                                            
                  mov PC, #0                       'Pointer auf Interrupttabelle IRQ 0
                  test IRQ, #%00000001    wz       'testen IRQ 0
         if_nz    andn IRQ, #%00000001 
                  test IRQ, #%00000010    wz       'testen IRQ 1
         if_nz    mov PC, #2
         if_nz    andn IRQ, #%00000010          
                  test IRQ, #%00000100    wz       'testen IRQ 2
         if_nz    mov PC, #4
         if_nz    andn IRQ, #%00000100                  
                  test IRQ, #%00001000    wz       'testen IRQ 3
         if_nz    mov PC, #6
         if_nz    andn IRQ, #%00001000                  
                  test IRQ, #%00010000    wz       'testen IRQ 4
         if_nz    mov PC, #8
         if_nz    andn IRQ, #%00010000                 
                  test IRQ, #%00100000    wz       'testen IRQ 5
         if_nz    mov PC, #10                   
         if_nz    andn IRQ, #%00100000
}
                  mov PC, #0
                  mov temp, #5
                  mov temp2, #%00000001
:ma1              test IRQ, temp2        wz
         if_nz    jmp #:ma2         
                  add PC, #2
                  shl temp2, #1
                  djnz :ma1, temp 
:ma2              andn IRQ, temp2

                  mov z8reg_ptr, z8reg_adr         'Register FA                     
                  add z8reg_ptr, #$FA
                  wrbyte IRQ, z8reg_ptr            'IRQ - Anforderung rcksetzen .
                 
                  mov nval, #0                     'ISR-Adresse holen
                  sub PC, #1                       '-1 wegen rd_xx
                  call #rd_xx                      'H-Teil Adresse einlesen
                  shl nval, #8
                  call #rd_xx                      'L-Teil Adresse einlesen
                  mov PC, nval                     'PC mit Adresse laden
                  
                  jmp #RD_Command
'.....................................................................................................
tcmx              xor val1, #$FF 
                  and val1, val2          wz
                  muxz FLAG, #%0100_0000
                  test val1, #$80         wz
                  muxz FLAG, #%0010_0000
                  and FLAG, #%1110_1111  
                  mov z8reg_ptr, z8reg_adr                          
                  add z8reg_ptr, #$FC
                  wrbyte FLAG, z8reg_ptr 
                  jmp #PCinc                  
'.....................................................................................................
daadd             mov temp, val1                    'Wert laden
                  shr temp, #4                      'und in Halbbytes
                  mov temp2, val1                   'extrahieren
                  and temp2,#$0F
                  test FLAG, #%1000_0000  wz        'teste Carry
                                                                       
    if_nz         cmp temp2, #$0A         wc        'wenn C=1 teste D3..D0 auf A..F
    if_nz_and_nc  mov val2, #$66                    'wenn C=1 und D3..D0 -> A..F, dann addiere 66
    if_nz_and_c   mov val2, #$60                    '                    -> 0..9, dann addiere 60
    if_nz         jmp #calc                         'wenn C=1 zurck

'...C=0...............................................................
                  cmp temp, #$09          wc        'teste D7..D4 
    if_nc         cmp temp2, #$0A         wc        'wenn D7..D4 -> 9..F teste D3..D0 -> A..F
    if_nc         mov val2, #$66                    'dann addiere 66
    if_nc         or FLAG, #%1000_0000              'setze C=1
    if_nc         jmp#calc                          'und zurck

                  cmp temp, #$0A          wc        'teste D7..D4  auf A..F
    if_nc         test FLAG, #%0000_0100  wz        'wenn ja, dann teste H-Flag
    if_nc_and_nz  mov val2, #$66                    'wenn A..F und H=1, addiere 66
    if_nc_and_z   mov val2, #$60                    'wenn A..F und H=0, addiere 60
    if_nc         or FLAG, #%1000_0000              'wenn A..F setze C=1    
    if_nc         jmp#calc                          'und zurck
 
                  cmp temp2, #$0A         wc        'teste D3..D0 auf A..F
                  test FLAG, #%0000_0100  wz        'teste H-Flag
    if_nc_or_nz   mov val2, #$06                    'wenn  A..F oder H=1 addiere 6
    if_nc_or_nz   jmp#calc                          'und zurck, sonst
                  mov val2, #$00                    'addiere 0
                  jmp#calc                          'und zurck
'.....................................................................................................  
dasub             mov val2, #0
                  test FLAG, #%1000_0000  wz
    if_z          test FLAG, #%0000_0100  wc
    if_z_and_c    mov val2, #$0A
    if_z_and_nc   mov val2, #$A0
    if_z          jmp #calc
    
                  test FLAG, #%0000_0100  wz
    if_z          mov val2, #$FA
                  jmp#calc
'.....................................................................................................
grp_Fx            cmp cmd, #$BF            wz      'iret
iret    if_z      jmp #iret_xx                  
                  cmp cmd, #$8F            wc      'illegal OP-Codes <8F
        if_c      jmp #PCinc                       '
                  cmp cmd, #$EF            wz      'ccf    
        if_z      xor FLAG, #%1000_0000 
                  cmp cmd, #$DF            wz      'scf    
        if_z      or FLAG, #%1000_0000
                  cmp cmd, #$CF            wz      'rcf    
        if_z      and FLAG, #%0111_1111
                  mov z8reg_ptr, z8reg_adr                     
                  add z8reg_ptr, #$FC                          
                  wrbyte FLAG, z8reg_ptr                       
                 
eidi              mov regnr, #$FB                  'ei, di                     - Cmd =< 9F
                  call #get_REGval
                  cmp cmd, #$9F            wz      'ei    
        if_z      or regval, #%1000_0000
                  cmp cmd, #$8F            wz      'di    
        if_z      and regval, #%0111_1111
                  jmp #set_end
'.....................................................................................................
cond              mov cmdhub_adr, flag_adr         'Basisadresse CMD-Sprungtabelle nach Zeiger <cmdhub_adr>  
                  mov temp, cmd_nr                                                                           
                  shl temp, #1                                                                               
                  add cmdhub_adr, temp             'und Offset zu Basisadresse im Hub addieren               
                  rdword temp2, cmdhub_adr                                                                   
                  mov temp3, FLAG                  'Flag nach <temp3>                                        
                  and temp3, temp2                 'maskieren nicht benutzter Flags                          
                  shr temp2, #8
                  cmp cmd_nr, #$08           wz    'bei unbedingtem Sprung
    if_z          jmp #cond_end                    'zurck zur Auswertung, sonst                                                                             
                  test cmd_nr, #%0000_0100   wz    'Test ob cmd 4x...7x, Cx...Fx                                                          
    if_nz         jmp #cond_end                    'wenn ja Sprung zur Auswertung , sonst
                                                   'Berechnung der kombinierten Bedingungen
                  test FLAG, #%0011_0000     wc
                  muxc temp3, #%0000_0001          'S xor V       - lt, ge
                  test FLAG, #%0100_0000     wz
    if_nz_or_c    or temp3, #%0000_0010            'Z or(S xor V) - le, gt
                  test FLAG, #%1100_0000     wz
                  muxnz temp3, #%0000_0100         'C or Z        - ule, ugt
                  jmp #cond_end                    'Sprung zur Auswertung    
                                                   
'......Flag-Bedingungen...............................................................................
'                    Wert     Maske
flagtable      word %00000000_00000000           '0  - n.d.      
               word %00000001_00000000           '1  - lt        
               word %00000010_00000000           '2  - le        
               word %00000100_00000000           '3  - ule       
               word %00010000_00010000           '4  - ov                                   
               word %00100000_00100000           '5  - mi                                   
               word %01000000_01000000           '6  - z                                    
               word %10000000_10000000           '7  - c                                                    
               word %00000000_00000000           '8  - unbedingt                                            
               word %00000000_00000000           '9  - ge                                            
               word %00000000_00000000           'A  - gt                                            
               word %00000000_00000000           'B  - ugt                                           
               word %00000000_00010000           'C  - nov                                           
               word %00000000_00100000           'D  - pl                                            
               word %00000000_01000000           'E  - nz                                            
               word %00000000_10000000           'F  - nc

'.....CMD-Gruppen-Sprungverteiler.......................................................................
'                   1.Aktion   2.Aktion         letzte Akt.  Parameter 1._2. Aktion
'                                                           (sdtyp|mode_sdtyp|mode )
grp_jmp        word rd_REG_x  ,cmdjmp_calc     ,0                ,$12_00            '0   
               word rd_REG_x  ,cmdjmp_calc     ,0                ,$13_00            '1   
               word r_right_x ,r_left_x        ,cmdjmp_calc      ,$21_11            '2   
               word r_right_x ,r_left_x        ,cmdjmp_calc      ,$22_11            '3   
               word rd_REG_x  ,rd_REG_x        ,cmdjmp_calc      ,$22_12            '4   
               word rd_REG_x  ,rd_REG_x        ,cmdjmp_calc      ,$12_23            '5     
               word rd_REG_x  ,rd_n            ,cmdjmp_calc      ,$12_60            '6   
               word rd_REG_x  ,rd_n            ,cmdjmp_calc      ,$13_60            '7   
               word rd_REG_x  ,REGcmd          ,set_end          ,$32_00            '8   
               word REGcmd    ,rd_REG_x        ,set_end          ,$01_01            '9   
               word REGcmd    ,set_REGvalminus ,grp_A            ,$01_00            'A   
               word condit    ,grp_B           ,0                ,$00_00            'B   
               word grp_C     ,0               ,0                ,$01_40            'C   
               word condit    ,grp_D           ,0                ,$00_00            'D   
               word REGcmd    ,grp_E           ,0                ,$11_00            'E   
               word grp_F     ,0               ,0                ,$00_00            'F   
                
'.....CMD-Sprungverteiler...............................................................................
'                 CMD-Jump    Aktion    Flags           
x0_jmp         word calc1   ,%100001001_0111000_   'dec           0    GRUPPE 0 
               word rlc     ,%001001001_1111000    'rlc           1
               word calc1   ,%100000001_0111000    'inc           2
               word jpIRR   ,%000000000_0000000    'jpIRR         3
               word da      ,%100000001_0110000    'da            4
               word pop     ,%000000000_0000000    'pop           5
               word com     ,%011011001_0110000    'com           6          
               word push    ,%000000000_0000000    'push          7
               word idw     ,%100001001_0111001    'decw          8
               word rl      ,%001001001_1111000    'rl            9
               word idw     ,%100000001_0111001    'incw          A
               word clrR    ,%000000000_0000000    'clr           B
               word rrc     ,%001010001_1111000    'rrc           C
               word sra     ,%001010001_1111000    'sra           D
               word rr      ,%001010001_1111000    'rr            E
               word swap    ,%001010001_0110000    'swap          F
'.......................................................................................................
               word calc1   ,%100001001_0111000_   'dec           0     GRUPPE 1
               word rlc     ,%001001001_1111000    'rlc           1
               word calc1   ,%100000001_0111000    'inc           2
               word srp     ,%000000000_0000000    'srp           3
               word da      ,%100000001_0110000    'da            4
               word pop     ,%000000000_0000000    'pop           5
               word com     ,%011011001_0110000    'com           6          
               word push    ,%000000000_0000000    'push          7
               word idw     ,%100001001_0111001    'decw          8
               word rl      ,%001001001_1111000    'rl            9
               word idw     ,%100000001_0111001    'incw          A
               word clrR    ,%000000000_0000000    'clr           B
               word rrc     ,%001010001_1111000    'rrc           C
               word sra     ,%001010001_1111000    'sra           D
               word rr      ,%001010001_1111000    'rr            E
               word swap    ,%001010001_0110000    'swap          F
'.......................................................................................................
               word addc    ,%100000001_1111010    'add           0      GRUPPE 2      
               word addc    ,%110010001_1111010    'adc           1              
               word subc    ,%100001001_1111010    'sub           2
               word subc    ,%110011001_1111010    'sbc           3
               word aox     ,%011010001_0110000    'or            4
               word aox     ,%011000001_0110000    'and           5
               word tcm     ,%011000000_0110000    'tcm           6
               word aox     ,%011000000_0110000    'tm            7
               word ldc_ci  ,dm_read               'lde r,Irr     8            
               word ldc_ci  ,dm_write              'lde Irr,r     9              
               word calc    ,%100001000_1111000    'cp            A
               word aox     ,%011011001_0110000    'xor           B
               word ldc_ci  ,pm_read               'ldc r,Irr     C
               word ldc_ci  ,pm_write              'ldc Irr,r     D
               word PCinc   ,%000000000_0000000    'no Opcode     E
               word PCinc   ,%000000000_0000000    'no Opcode     F
'.......................................................................................................
               word addc    ,%100000001_1111010    'add           0      GRUPPE 3 
               word addc    ,%110010001_1111010    'adc           1    
               word subc    ,%100001001_1111010    'sub           2    
               word subc    ,%110011001_1111010    'sbc           3    
               word aox     ,%011010001_0110000    'or            4    
               word aox     ,%011000001_0110000    'and           5    
               word tcm     ,%011000000_0110000    'tcm           6    
               word aox     ,%011000000_0110000    'tm            7    
               word ldc_ci  ,dm_read               'ldei Ir,Irr   8            
               word ldc_ci  ,dm_write              'ldei Irr,Ir   9              
               word calc    ,%100001000_1111000    'cp            A
               word aox     ,%011011001_0110000    'xor           B
               word ldc_ci  ,pm_read               'ldciIr,Irr    C
               word ldc_ci  ,pm_write              'ldci Irr,Ir   D
               word calc    ,%101000001_0000000    'ld r,IR       E
               word ldIrrF3 ,%000000000_0000000    'ld Ir,r       F
'.......................................................................................................
               word addc    ,%100000001_1111010    'add           0      GRUPPE 4
               word addc    ,%110010001_1111010    'adc           1    
               word subc    ,%100001001_1111010    'sub           2    
               word subc    ,%110011001_1111010    'sbc           3    
               word aox     ,%011010001_0110000    'or            4    
               word aox     ,%011000001_0110000    'and           5    
               word tcm     ,%011000000_0110000    'tcm           6    
               word aox     ,%011000000_0110000    'tm            7    
               word PCinc   ,%000000000_0000000    'no opcode     8            
               word PCinc   ,%000000000_0000000    'no opcode     9             
               word calc    ,%100001000_1111000    'cp            A
               word aox     ,%011011001_0110000    'xor           B
               word cmd1    ,%000000000_0000000    'no opcode     C
               word callIRR ,%000000000_0000000    'call_IRR      D
               word calc    ,%101000001_0000000    'ld_RR         E
               word PCinc   ,%000000000_0000000    'no opcode     F
'.......................................................................................................
               word addc    ,%100000001_1111010    'add           0      GRUPPE 5
               word addc    ,%110010001_1111010    'adc           1    
               word subc    ,%100001001_1111010    'sub           2    
               word subc    ,%110011001_1111010    'sbc           3    
               word aox     ,%011010001_0110000    'or            4    
               word aox     ,%011000001_0110000    'and           5    
               word tcm     ,%011000000_0110000    'tcm           6    
               word aox     ,%011000000_0110000    'tm            7    
               word PCinc   ,%000000000_0000000    'no opcode     8            
               word PCinc   ,%000000000_0000000    'no opcode     9              
               word calc    ,%100001000_1111000    'cp            A
               word aox     ,%011011001_0110000    'xor           B
               word PCinc   ,%000000000_0000000    'no opcode     C
               word PCinc   ,%000000000_0000000    'no opcode     D
               word calc    ,%101000001_0000000    'ldR_IR        E
               word ldIRRF5 ,%000000000_0000000    'ldIR_R        F
'.......................................................................................................
               word addc    ,%100000001_1111010    'add           0      GRUPPE 6
               word addc    ,%110010001_1111010    'adc           1    
               word subc    ,%100001001_1111010    'sub           2    
               word subc    ,%110011001_1111010    'sbc           3    
               word aox     ,%011010001_0110000    'or            4    
               word aox     ,%011000001_0110000    'and           5    
               word tcm     ,%011000000_0110000    'tcm           6    
               word aox     ,%011000000_0110000    'tm            7    
               word PCinc   ,%000000000_0000000    'no opcode     8            
               word PCinc   ,%000000000_0000000    'no opcode     9              
               word calc    ,%100001000_1111000    'cp            A
               word aox     ,%011011001_0110000    'xor           B
               word PCinc   ,%000000000_0000000    'no opcode     C
               word calladr ,%000000000_0000000    'calladr       D
               word calc    ,%101000001_0000000    'ldR_n         E
               word PCinc   ,%000000000_0000000    'no opcode     F
'.......................................................................................................
               word addc    ,%100000001_1111010    'add           0      GRUPPE 7
               word addc    ,%110010001_1111010    'adc           1    
               word subc    ,%100001001_1111010    'sub           2    
               word subc    ,%110011001_1111010    'sbc           3    
               word aox     ,%011010001_0110000    'or            4    
               word aox     ,%011000001_0110000    'and           5    
               word tcm     ,%011000000_0110000    'tcm           6    
               word aox     ,%011000000_0110000    'tm            7    
               word PCinc   ,%000000000_0000000    'no opcode     8            
               word PCinc   ,%000000000_0000000    'no opcode     9              
               word calc    ,%100001000_1111000    'cp            A
               word aox     ,%011011001_0110000    'xor           B
               word ldrX    ,%000000000_0000000    'ldr_X         C
               word ldrX    ,%000000000_0000000    'ldX_r         D
               word calc    ,%101000001_0000000    'ldIR_n        E
               word PCinc   ,%000000000_0000000    'no opcode     F
               
'.....Definition TINY- Speicher ........................................................................
REGx           byte $00 [$100]
ROMx           file "2kprop.bin" 
RAMx           byte $00 [ram_len]
KEYx           byte $00 [key_len +1]
col_base       word $00 [lencol]
bitmap_base    long $00 [lenbmp] 

DAT  
org 0
'******************************************************************************************************
'*---- Z8 Command- Cog -------------------------------------------------------------------------------*
'******************************************************************************************************  
START             mov z8bus_adr, Par               'Adresse Bus  einlesen

reset             mov hubadr, do_res_adr
                  jmp #vm
                  
'------Hauptschleife-----------------------------------------------------------------------              
cmd1              rdlong temp2, z8bus_adr          'Bus einlesen
                  test temp2, reset_mask  wz       'Testen auf Reset (Reset=1)
        if_nz     jmp #reset                       'Sprung, wenn ja

'-----------------------------------------------------------------------------------------------                  
'......Systemregister einlesen..................................................................
                 mov regnr, #$F9                    'einlesen Interruptregister, FLAG,RP,SP   
                 mov z8reg_ptr, z8reg_adr
                 add z8reg_ptr, regnr
                 movd :arg,#IPR     
                 mov temp,#7       
:arg             rdbyte IPR,z8reg_ptr 
                 add :arg,d0       
                 add z8reg_ptr, #1 
                 djnz temp,#:arg
                 mov SP, SPH
                 shl SP, #8
                 or SP, SPL   
                 
'-----------------------------------------------------------------------------------------------              
'......Interruptverarbeitung...............................................................
                  test IRQ, #%00111111    wz       'Interruptanforderung vorhanden?
         if_z     jmp #RD_Command                  'Sprung, wenn nicht, sonst         
                  test IMR, #%10000000    wz       'Interrupts global freigegeben?
         if_z     jmp #RD_Command                  'Sprung zurck,wenn nicht, sonst
                  and IMR, IRQ           nr, wz    'Interrupt entspricht Maskierung?
         if_z     jmp #RD_Command                  'Sprung zurck,wenn nicht, sonst
'......Interruptannahme....................................................................
                  mov hubadr, do_int_adr           'wenn ja, dann
                  jmp #vm                          'Verarbeitung, sonst weiter
                  
'------einlesen PC, 1.Byte Befehl(Kommando) ----------------------------------------------------

RD_Command        call #pm_read                    'Bus einlesen
                  add adr_val, #1
                  rdbyte arg1, adr_val             'look ahead nchstes Byte als Argument lesen
                  add adr_val, #1
                  rdbyte arg2, adr_val             'look ahead nchstes Byte als Argument lesen 
                  mov  cmd, temp                   'Daten => Cmd
                  mov  cmd_grp, cmd                '
                  and  cmd_grp, #$0F               'Cmd-Gruppe extrahieren
                  mov  cmd_nr, cmd
                  shr  cmd_nr, #4                  'Cmd-Nummer extrahieren
 
'.....CMD-Sprung Adresse berechnen......................................................................
                  mov ival, #0
                  mov cmdhub_adr, grp_jmp_adr      'Basisadresse Gruppen-Sprungverteiler nach Zeiger <cmdhub_adr>
                  mov temp, cmd_grp                'Kommandogruppe fr weitere Berechnung
                  shl temp, #1
                  call #rd_table                   'UP-Adressen nach temp2, temp3 einlesen
                  movs :ma1, temp2
                  movs :ma2, temp3
                  call #rd_next                    'einlesen
                  movs :ma3, temp3
                  add cmdhub_adr, #2
                  rdbyte mdsel1,cmdhub_adr
                  add cmdhub_adr, #1
                  rdbyte mdsel2,cmdhub_adr
                  mov mode, mdsel2
                  and mode, #$0F
                  mov sd_typ, mdsel2 
                  shr sd_typ, #4   
:ma1              call #rd_REG                     'UP's aus Tabelle ausfhren !!modifizierter Befehl!!
                  mov mode, mdsel1
                  and mode, #$0F
                  mov sd_typ, mdsel1 
                  shr sd_typ, #4 
:ma2              call #rd_REG                     '!!Alle in Tabelle verwendeten UP's mssen mit rd_REG_ret-Marke  _
:ma3              call #rd_REG                     '  bei ret enden oder aber Sprung!
'-----------------------------------------------------------------------------------------------------------                   
cmdjmp_calc       mov cmdhub_adr, x0_adr           'Basisadresse CMD-Sprungtabelle nach Zeiger <cmdhub_adr>
                  mov temp, cmd_grp                'Offset berechnen
                  shl temp, #4
                  or temp, cmd_nr
                  call #rd_table                   'einlesen aus Tabelle
                  mov cmdjmp, temp2                'Adresse UP fr Cmd nach <cmdjmp>
                  mov action, temp3                'temp3 enthlt ALU-Aktion und Flagmaske oder
                  mov todo, temp3                  'Adresse fr UP
                  mov flag_mask, action            'Flagmaske extrahieren
                  shl flag_mask, #1
                  and flag_mask, #$FF
                  shr action, #7   
                  jmp cmdjmp                       'Sprung zu Adresse im Verteiler
'........................................................................................................
rd_table          shl temp, #2                     'vervierfachen wegen 2*Word
                  add cmdhub_adr, temp             'und Offset zu Basisadresse im Hub addieren
                  rdword temp2, cmdhub_adr
rd_next           add cmdhub_adr, #2
                  rdword temp3, cmdhub_adr
rd_table_ret
rd_next_ret       ret

'******************************************************************************************************                   
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe xF   - div. Programmsteuerung
'---------------------------------------------------------------------------------------------------------
grp_F             cmp cmd, #$FF              wz    'nop
        if_z      jmp #PCinc
                  cmp cmd, #$AF              wz    'ret    
ret_up  if_z      jmp #ret_xx
                  mov hubadr, grp_Fx_adr
                  jmp #vm
'.........................................................................................................
ret_xx            call #ST2PC                      'PC vom Stack holen
                  jmp #cmd1
'.........................................................................................................
iret_xx           call #ret_flag                   'Flag und PC vom Stack holen 
                  or IMR, #%1000_0000  
                  mov z8reg_ptr, z8reg_adr                           
                  add z8reg_ptr, #$FB
                  wrbyte IMR, z8reg_ptr 
                  jmp #cmd1
                                                                                                    
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe xE   - inc r
'---------------------------------------------------------------------------------------------------------
grp_E             mov action, #%100000001          'add                                                 ok
                  mov flag_mask, #%0111_0000       'Z,S,V-Flag
'......................................................................................................      
calc1             mov val2, #$01                   'Operationen mit '1'

'..... Arithmetik- Routine (val1, val2)................................................................
calc              test FLAG, #%1000_0000      wc   'Carry Z8 -> Carry Prop fr Berechnungen mit C
                  test action, #%000000001    wz   'Result schreiben?
                  or action, #%000000001           'immer Result schreiben setzen, wegen Flagberechnung CSV
                  and val2, #$FF
                  call #calcx    
           
        if_nz     mov tempval, resultx             'schreibe Result, wenn gesetzt
        if_z      mov tempval, val1                'sonst ist Result der urspr. Wert
                  
'......LMM-VM fr Flagberechnung.......................................................................
                  tjz flag_mask, #calc_end         'wenn Cmd ohne Flagbeeinflussung, dann Sprung
                                                   'sonst Flagberechnung per LMM
                                                   
                  test flag_mask, #%0000_0010  wz  'Test auf 8/16-bit Arithmetik
        if_nz     mov temp, flag16_calc            '16-bit Flag's          
        if_z      mov temp, flag8_calc             '8-bit Flag's
        
              
'......LMM-VM .........................................................................................              
                  mov hubadr, temp                 '
vm                rdlong instr, hubadr
                  add hubadr, #4
instr             nop                                    
                  jmp #vm
'......................................................................................................
calcx             movi :ma1, action
                  mov resultx, val1
:ma1              add resultx, val2                'modifizierter Befehl!!      
calcx_ret         ret
'......................................................................................................
calc_end          mov regval, tempval
dst_end           and regval, #$ff
set_end2          mov regnr, dst_regnr 
set_end           call #set_REGval                    
                  jmp #PCinc

'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe xD   - jmp cc,nn
'---------------------------------------------------------------------------------------------------------
grp_D  '           cmp cmd_nr, #1      wc           'illegaler Opcode 0B? (verwendet in ISR des TINY)
       '  if_c     jmp #PC3inc                      'Sprung, wenn ja

         if_z     jmp #jump_abs                    'wenn erfllt, springen
PC3inc            add PC, #1                       'sonst PC auf nchsten Befehl setzen (3x PC+1)
PC2inc            add PC, #1                       'Programmzhler + 1  
PCinc             add PC, #1                       'Programmzhler + 1
                  jmp #cmd1                        'und zurck  
                  
jump_abs          shl arg1, #8
                  or arg1, arg2
                  mov PC, arg1 
                  jmp #cmd1
 
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe xC   - ld r,n
'---------------------------------------------------------------------------------------------------------
grp_C             mov regnr, cmd_nr
                  or regnr, RP
                  mov regval, arg1
                  add PC, #1
                  jmp #set_end
                  
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe xB   - jr cc,nn
'---------------------------------------------------------------------------------------------------------
grp_B   '          cmp cmd_nr, #1      wc           'illegaler Opcode 0B? (verwendet in ISR des TINY)
        ' if_c     jmp #PC2inc                      'Sprung, wenn ja
         if_nz    jmp #PC2inc
jump_rel          test arg1, #$80     wz
         if_z     add PC, #2
         if_nz    sub PC, #$FE 
                  add PC, arg1 
                  jmp #cmd1 
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe xA   - djnz r,n
'---------------------------------------------------------------------------------------------------------
grp_A             tjnz regval, #jump_rel       
                  jmp #PC2inc
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x9   - ld R,r
'---------------------------------------------------------------------------------------------------------
 '--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x8   - ld r,R
'---------------------------------------------------------------------------------------------------------
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x7   - IR,n
'---------------------------------------------------------------------------------------------------------
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x6   - R,n
'---------------------------------------------------------------------------------------------------------
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x5   - R,IR
'---------------------------------------------------------------------------------------------------------
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x4   - R,R
'---------------------------------------------------------------------------------------------------------
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x3   - r,IR
'---------------------------------------------------------------------------------------------------------
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x2   - r,r
'---------------------------------------------------------------------------------------------------------
 '--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x1   - IR
'---------------------------------------------------------------------------------------------------------
'--------------------------------------------------------------------------------------------------------- 
'CMD-Gruppe x0   - R               
'---------------------------------------------------------------------------------------------------------

'*********************************************************************************************************

'--------------------------------------------------------------------------------------------------------- 
'CMD-Aktionen ********************************************************************************************
'---------------------------------------------------------------------------------------------------------
ldIrrF3           mov regnr, val1
                  mov regval, src_regnr
                  jmp #set_end
'......................................................................................................          
idW               call #Irr
                  mov val1, regval
                  jmp #calc1
'......................................................................................................                                                        '
clrR              mov regval, #0                      '                                         
                  jmp #set_end
'......................................................................................................
pop               call #ST_read                       'Stack nach <temp>
                  mov regval, temp
                  jmp #set_end2
'......................................................................................................
push              mov temp, regval                    '                                          
                  call #ST_write
                  jmp #PCinc
'......................................................................................................             
calladr           add PC, #1
                  call #PC2ST
                  jmp #jump_abs '2
'......................................................................................................
callIRR           call #PC2ST
                  mov regnr, src_regnr
                  jmp #jpIRR2
jpIRR             mov regnr, dst_regnr
jpIRR2            call #get_IRR
jpIRR3            mov PC, regval
                  jmp #cmd1
'......................................................................................................
srp               mov RP, nval                        'Registerpointer setzen
                  mov regval, RP
                  mov regnr, #$FD
                  jmp #set_end                        'und zurck
'......................................................................................................
addc              andn FLAG, #%00001000 
                  jmp #subcc
'......................................................................................................                  
subc              or FLAG, #%00001000
subcc             mov z8reg_ptr, z8reg_adr 
                  add z8reg_ptr, #$FC      
                  wrbyte FLAG, z8reg_ptr   
                  jmp #calc
'......................................................................................................                   
swap              shl val1, #8                                                                   
                  or val1, regval
                  mov val2, #4                  
                  jmp #calc
'......................................................................................................   
rlc               test FLAG,  #%1000_0000       wz    'Teste C-Flag                             
                  jmp #rl1
'......................................................................................................
rl                test val1, #%1000_0000        wz    'Teste bit7                                
rl1               muxnz val1, b31_mask                'schreibe nach bit31
                  jmp #calc1                                                                               
'......................................................................................................
rrc               test FLAG,  #%1000_0000       wz    'Teste C-Flag -> z                         
                  test val1, #%0000_0001        wc    'Teste bit0   -> c
                  jmp #rr1
'......................................................................................................
rr                test val1, #%0000_0001      wz, wc  'Teste bit0   -> z,c                        
rr1               muxnz val1, b8_mask                 'schreibe z nach bit8
                  muxc val1, b9_mask                  'schreibe c nach bit9
                  jmp #calc1
'......................................................................................................
sra               test val1, #%0000_0001        wc    'Teste bit0   -> c                         
                  test val1, #%1000_0000        wz    'Teste bit7
                  and FLAG, #%1110_1111 
                  jmp #rr1
'......................................................................................................          
com               test val1, #%1000_0000        wz    'Teste bit7                                 
                  muxnz val1, b8_mask                 'schreibe z nach bit8  
                  mov val2, #$ff
aox               and FLAG, #%1110_1111
                  jmp #calc
'......................................................................................................
tcm               mov hubadr, tcmx_adr
                  jmp #vm
'......................................................................................................            
ldc_ci            mov hubadr, ldcx_adr
                  jmp #vm
                                                                                                                                 
ldcci             movs :ma, todo                      'Aktion setzen                                                                                        
                  mov temp, val1                                                                                                                            
:ma               call #pm_read                       'Lesen/Schreiben PM bzw. DM                                                                           
                  mov PC, tempval                     'PC zurckholen                                                                                       
                  mov regval, temp
                  jmp #dst_end
'.....................................................................................................                      
ldIRRF5           mov hubadr, ldIRR5_adr
                  jmp #vm
'......................................................................................................             
ldrX              mov hubadr, ldxx_adr
                  jmp #vm
'......................................................................................................                   
da                test  FLAG, #%0000_1000    wz    'D-Flag testen
         if_z     mov hubadr, daadd_adr
         if_nz    mov hubadr, dasub_adr
                  jmp #vm
                                
'---------------------------------------------------------------------------------------------------------

'*********************************************************************************************************
'* Common-Parts  
'*********************************************************************************************************
inc_Irr          add regval, #1
set_Irr          ror regval, #8
                 call #set_REGval
                 rol regval, #8
                 call #set_incREGval
inc_Irr_ret
set_Irr_ret      ret
'---------------------------------------------------------------------------------------------------------    
get_Irr          call #get_REGval
Irr              mov tempval, regval
Irr2             shl tempval, #8
                 call #get_incREGval
                 or regval, tempval
get_Irr_ret
Irr2_ret
Irr_ret          ret
'---------------------------------------------------------------------------------------------------------
condit           mov hubadr, cond_adr
                 jmp #vm
                         
cond_end         cmp temp3, temp2    wz           'vergleichen
                 jmp #all_end
'---------------------------------------------------------------------------------------------------------
set_REGvalminus  sub regval, #1      wz
                 jmp #set_REGval
set_incREGval    add regnr, #1
set_REGval       mov z8reg_ptr, z8reg_adr
                 add z8reg_ptr, regnr
                 wrbyte regval, z8reg_ptr
                 jmp #setreg_end 
'---------------------------------------------------------------------------------------------------------
get_incREGval    add regnr, #1
                 jmp #get_REGval
'---------------------------------------------------------------------------------------------------------
REGcmd           mov regnr, cmd_nr 
                 or regnr, RP  
                 jmp # abs_REG2
'.......................................................                 
r_right_x        mov nval, #0  
                 mov nval, arg1
                 add PC, #1
r_right_y        mov regnr, nval
r_right          and regnr, #$0F
                 jmp #rr2
r_left_x         mov regnr, nval
r_left           shr regnr, #4       
rr2              or regnr, #$E0  
                 jmp # abs_REG
'---------------------------------------------------------------------------------------------------------
rd_REG_x     
rd_n2            mov nval, #0
rd_n             shl nval, #8
rd_REG           call #rd_xx
                 tjz mode, #reg_end
                 sub mode, #1
                 mov regnr, temp
abs_REG          mov temp, regnr
                 and temp, #$F0
                 cmp temp, #$E0     wz
          if_z   and regnr, #$0F
          if_z   or  regnr, RP
                 shl ival, #8
                 or ival, regnr
abs_REG2         tjz mode, #reg_end
                 sub mode, #1 
get_REGval       mov z8reg_ptr, z8reg_adr
                 add z8reg_ptr, regnr
                 rdbyte regval, z8reg_ptr
                 tjz mode, #reg_end
                 mov regnr, regval
                 sub mode, #1
                 jmp #get_REGval 
reg_end          test sd_typ, #%0000_0100  wz
          if_nz  mov regval, nval 
                 test sd_typ, #%0000_0001  wz
          if_nz  mov dst_regnr, regnr
          if_nz  mov val1, regval
                 test sd_typ, #%0000_0010  wz
          if_nz  mov src_regnr, regnr
          if_nz  mov val2, regval 
                 mov mode, #0
                 mov sd_typ, #0
all_end          nop
setreg_end
condit_ret
get_REGval_ret
get_incREGval_ret
r_left_x_ret
r_left_ret
r_right_x_ret
r_right_y_ret
r_right_ret
REGcmd_ret
rd_n2_ret
rd_n_ret
rd_REG_x_ret
set_REGvalminus_ret
set_incREGval_ret
set_REGval_ret          
rd_REG_ret          
abs_REG_ret       ret
'.........................................................................................................
rd_xx            add PC, #1
                 call #pm_read
                 or nval, temp
rd_xx_ret        ret

'---------------------------------------------------------------------------------------------------------

'*********************************************************************************************************
'* Routinen Schreiben / Lesen vom Bus / Stack / Port2
'*********************************************************************************************************

'-----Stackoperationen------------------------------------------------------------------------------------
   
ret_flag          call #ST_read                    'vom Stack lesen
                  mov FLAG, temp                   'nach Flag schreiben
ST2PC             call #ST_read                    'vom Stack lesen  (H-Teil Adesse)
                  shl temp, #8                     'verschieben und
                  mov adr,temp                     'nach <arg>
                  call #ST_read                    'vom Stack lesen  (L-Teil Adresse)
                  or adr,temp                      'zu <arg> hinzufgen
                  mov PC,adr                       'PC mit <arg> setzen
ret_flag_ret
ST2PC_ret         ret
                  
'--------------------------------------------------------------------------------------------------------- 
ST_read           mov temp3, PC
                  mov PC, SP
                  call #dm_read
                  add SP, #1
                  jmp #ST_common
'---------------------------------------------------------------------------------------------------------
PC2ST             mov temp, PC
                  call #STup
                  mov temp, PC
                  shr temp, #8
ST_write          call #STup
ST_common         and temp, #$FF
                  mov PC,temp3
                  mov regnr, #$FD
                 
set_SP            mov regval, SP
                  ror regval, #8
                  call #set_incREGval
                  rol regval, #8
                  call #set_incREGval
PC2ST_ret
ST_read_ret                    
ST_write_ret        
set_SP_ret        ret

'.........................................................................................................                  
STup              mov temp3, PC
                  sub SP, #1        
                  mov PC, SP
                  call #dm_write
                  mov PC, temp3
STup_ret          ret

'*********************************************************************************************************                           
'-----Speicheroperationen---------------------------------------------------------------------------------
'*********************************************************************************************************
dm_read           mov rw_mask, #0
                  jmp #DM
dm_write          mov rw_mask, #1
DM                cmp PC, dmem_s          wc        'Test ob DM-Adresse
        if_nc     cmp dmem_e, PC          wc        'innerhalb Bereich
        if_c      jmp #NOMEM                        'wenn nicht, dann kein Speicher
                  jmp #PM                           'sonst weiter wie PM
'.........................................................................................................
pm_read           mov rw_mask, #0
                  jmp #PM
pm_write          mov rw_mask, #1
'.........................................................................................................
PM  
'.....ROM-Adressen........................................................................................
                  cmp PC, rom_e            wc        'Vgl. Adresse mit ROM-Ende
        if_c      mov adr_val, rombaseadr 
        if_c      add adr_val, PC
        if_c      jmp #ROM
        
'.....RAM-Adressen........................................................................................        
                  mov adr_val, PC                    'vgl. RAM-Anfang u. log. Adresse minus log. Startadress RAM  
                  cmpsub adr_val, ram_s    wc        '-> Offset fr phys. Adresse
        if_c      add adr_val, rambaseadr            'physische Adresse im HubRam errechnen
        if_c      jmp #RW  

'.....Tastatur-Adressen...................................................................................   
                  cmp PC, key_e            wc        'vgl. mit Tastatur hchste Adresse
        if_c      cmp key_s, PC            wc        'wenn kleiner, dann vgl. mit niedrigster Adresse
        if_c      mov adr_val, PC  
        if_c      sub adr_val, key_s
        if_c      add adr_val, keybaseadr
        if_c      jmp #KEY                           'wenn innerhalb des Bereiches dann Sprung, sonst
        
'.....sonstige Memory-Mapped I/O..........................................................................        
        
'.....kein Speicher.......................................................................................        
NOMEM             test rw_mask, #1         wz        'Test ob RD oder WR
        if_nz     jmp #go_end                        'bei  Schreiben zurck
                  mov temp, #$FF                     'kein Speicher
                  jmp #go_end

'.....Tastatur-Abfrage....................................................................................
KEY               rdbyte temp, adr_val               'Wert(Byte) von phys. Adresse im HUB einlesen 
                  mov temp2, #$0
                  wrbyte temp2, adr_val              '0 nach phys. Adresse im HUB schreiben
                  jmp #go_end

'.....ROM - Schreibschutz.................................................................................
ROM               test rw_mask, #1         wz        'Test ob RD oder WR
        if_nz     jmp #go_end                        'bei Schreibzugriffen Ende-Sprung -> Schreibschutz   
                  jmp #RD                            'sonst zu RD springen 

'.....Speicher schreiben/lesen............................................................................          
RW                test rw_mask, #1         wz        'Test ob RD oder WR
        if_nz     wrbyte temp, adr_val
RD      if_z      rdbyte temp, adr_val               'Wert(Byte) von phys. Adresse im HUB einlesen    
       
'.........................................................................................................
go_end
pm_read_ret
dm_read_ret
pm_write_ret
dm_write_ret      ret

'*********************************************************************************************************
'---- Data & Variable ------------------------------------------------------------------------------------
'*********************************************************************************************************
 d0            long  $200                   
 z8bus_adr     long  0
 z8reg_adr     long  0
 reset_mask    long  %10000000_00000000_00000000_00000000
 adr_mask      long  %00000000_00000000_11111111_11111111 
 cmd           long  0
 arg1          long  0
 arg2          long  0
 dst_regnr     long  0
 src_regnr     long  0

 IPR           long  0
 IRQ           long  0
 IMR           long  0
 FLAG          long  0
 RP            long  0
 SPH           long  0
 SPL           long  0
 SP            long  0
 PC            long  0   
 
 TMR           long  0
 T1            long  0           
 PRE1          long  0         
 T2            long  0           
 PRE2          long  0         
 
 flag8_calc    long  0
 flag16_calc   long  0
 setF0_7_adr   long  0
 ldxx_adr      long  0
 ldcx_adr      long  0 
 ldIRR5_adr    long  0
 daadd_adr     long  0
 dasub_adr     long  0
 tcmx_adr      long  0
 grp_Fx_adr    long  0
 do_res_adr    long  0
 do_int_adr    long  0
 cond_adr      long  0
 flag_adr      long  0
 x0_adr        long  0
 grp_jmp_adr   long  0 
 z8reg_ptr     long  0

 regnr         long  0
 regval        long  0
 v_16          long  $0000_8000
 z_16          long  $0000_FFFF
 c_16          long  $0001_0000
 b8_mask       long  %01_0000_0000
 b9_mask       long  %10_0000_0000
 b31_mask      long  %10000000_00000000_00000000_00000000
 rombaseadr    long  @ROMx
 rambaseadr    long  @RAMx
 keybaseadr    long  @KEYx
 rom_s         long   romstart       
 rom_e         long   romend          
 ram_s         long   ramstart        
 ram_e         long   ramend        
 key_s         long   keystart      
 key_e         long   keyend +1     
 dmem_s        long   dmemstart      
 dmem_e        long   dmemend
                                                                           
Mem            res   1                                                 
temp           res   1                                                 
temp2          res   1                                                 
temp3          res   1
temp4          res   1
tempval        res   1
templong       res   1
rw_mask        res   1                                                 
flag_mask      res   1
nval           res   1
ival           res   1                                                 
val1           res   1                    
val2           res   1
adr            res   1
adr_val        res   1 
resultx        res   1                    
action         res   1                    
hubadr         res   1
cmd_grp        res   1
cmd_nr         res   1        
cmdjmp_adr     res   1
cmdhub_adr     res   1
todo           res   1
cmdjmp         res   1
grpjmp         res   1
mdsel1         res   1
mdsel2         res   1
mode           res   1
sd_typ         res   1
hubtemp        res   1
'..............................
              
fit                                       
                                          